import 'package:chhota_stocks_flutter/l10n/translations.dart';
import 'package:chhota_stocks_flutter/src/core/functions/functions.dart';
import 'package:chhota_stocks_flutter/src/feature/commonWidget/common_widgets.dart';
import 'package:chhota_stocks_flutter/src/feature/commonWidget/src/expansion_animation.dart';
import 'package:chhota_stocks_flutter/src/feature/portfolio/bloc/aggregated_bloc/aggregated_cubit.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

class PortfolioCard extends StatelessWidget {
  final bool isOverview;
  const PortfolioCard({Key? key, this.isOverview = false}) : super(key: key);
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<AggregatedCubit, AggregatedState>(
        builder: (context, state) {
      if (state is AggregatedSuccess && isOverview) {
        return _PortfolioCardOverview(
          currentReturnPercent: state.aggregate.currentReturnsPercent,
          currentInvestment: state.aggregate.currentInvestment,
          currentReturnAmount: state.aggregate.currentReturns,
          totalReturnAmount: state.aggregate.totalReturns,
          totalReturnPercent: state.aggregate.totalReturnsPercent,
          currentValue: state.aggregate.networth,
        );
      } else if (state is AggregatedSuccess) {
        return _PortfolioCard(
          invested: state.aggregate.currentInvestment,
          current: state.aggregate.networth,
          plPercent: state.aggregate.currentReturnsPercent,
          plAmt: state.aggregate.currentReturns,
        );
      } else if (state is AggregatedLoading) {
        return const Padding(
          padding: EdgeInsets.symmetric(horizontal: 20.0),
          child: ShimmerLoader(),
        );
      }

      return const _PortfolioCardOverview();
    });
  }
}

class _PortfolioCard extends StatelessWidget {
  final num? invested;
  final num? current;
  final num? plPercent;
  final num? plAmt;
  const _PortfolioCard(
      {Key? key, this.invested, this.current, this.plPercent, this.plAmt})
      : super(key: key);
  @override
  Widget build(BuildContext context) {
    return Card(
      shadowColor: Colors.grey,
      shape: OutlineInputBorder(
          borderRadius: BorderRadius.circular(10),
          borderSide: const BorderSide(color: Colors.black12)),
      margin: const EdgeInsets.all(8),
      elevation: 4.0,
      child: Padding(
        padding: const EdgeInsets.all(12.0),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            const SizedBox(
              height: 8.0,
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                _ColumnText(
                  heading: "Invested",
                  subtitle:
                      "₹ ${invested != null ? indiaFormat.format(invested).replaceAll("INR", "") : "0.0"}",
                ),
                _ColumnText(
                  crossAxisAlignment: CrossAxisAlignment.end,
                  heading: "Current",
                  subtitle:
                      "₹ ${current != null ? indiaFormat.format(current).replaceAll("INR", "") : "0.0"}",
                ),
              ],
            ),
            const SizedBox(
              height: 12.0,
            ),
            const Divider(
              color: Colors.white,
              height: 4,
            ),
            const SizedBox(
              height: 12.0,
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                const Text(
                  "P&L",
                  style: TextStyle(fontWeight: FontWeight.w300, fontSize: 14.0),
                ),
                Row(
                  children: [
                    Text(
                      "₹ ${plAmt != null ? indiaFormat.format(plAmt).replaceAll("INR", "") : " 0.0"}",
                      style: TextStyle(
                          color: plAmt != null && plAmt!.isNegative
                              ? Colors.red
                              : Colors.green,
                          fontWeight: FontWeight.w500,
                          fontSize: 18.0),
                    ),
                    const SizedBox(
                      width: 4.0,
                    ),
                    if (plPercent != null)
                      Text(
                        plPercent!.toStringAsFixed(2) + " %",
                        style: TextStyle(
                            color: plPercent!.isNegative
                                ? Colors.red
                                : Colors.green,
                            fontWeight: FontWeight.w400,
                            fontSize: 12.0),
                      ),
                  ],
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }
}

class _PortfolioCardOverview extends StatefulWidget {
  final num? currentValue;
  final num? totalReturnPercent;
  final num? totalReturnAmount;
  final num? currentInvestment;
  final num? currentReturnPercent;
  final num? currentReturnAmount;
  const _PortfolioCardOverview(
      {Key? key,
      this.currentValue,
      this.totalReturnPercent,
      this.totalReturnAmount,
      this.currentInvestment,
      this.currentReturnPercent,
      this.currentReturnAmount})
      : super(key: key);

  @override
  State<_PortfolioCardOverview> createState() => _PortfolioCardOverviewState();
}

class _PortfolioCardOverviewState extends State<_PortfolioCardOverview> {
  bool isExpanded = false;
  @override
  Widget build(BuildContext context) {
    return Card(
      color: Colors.grey[100],
      // shadowColor: Colors.grey,
      shape: OutlineInputBorder(
        borderRadius: BorderRadius.circular(10),
        borderSide: const BorderSide(color: Colors.transparent),
      ),
      margin: const EdgeInsets.symmetric(horizontal: 18),
      // elevation: 1.0,
      child: Padding(
        padding: const EdgeInsets.all(18.0),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Text(
                  Translations.of(context).overview,
                  //"Overview",

                  overflow: TextOverflow.ellipsis,
                  style: TextStyle(fontSize: 16.0, fontWeight: FontWeight.bold),
                ),
                Container(
                  width: 25,
                  alignment: Alignment.center,
                  child: TextButton(
                    style: ButtonStyle(
                        overlayColor:
                            MaterialStateProperty.all(Colors.transparent)),
                    onPressed: () {
                      setState(() {
                        isExpanded ^= true;
                      });
                    },
                    child: AnimatedSwitcher(
                      duration: const Duration(milliseconds: 500),
                      switchInCurve: Curves.fastOutSlowIn,
                      switchOutCurve: Curves.fastOutSlowIn,
                      child: Icon(
                        isExpanded
                            ? Icons.keyboard_arrow_up_outlined
                            : Icons.keyboard_arrow_down_outlined,
                        color: Colors.black.withOpacity(0.9),
                        size: 25,
                      ),
                    ),
                  ),
                ),
              ],
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                _ColumnText(
                    heading:Translations.of(context).currentVal,
                    //"Current Value",
                    subtitle:
                        "₹ ${widget.currentValue != null ? indiaFormat.format(widget.currentValue).replaceAll("INR", "") : "--"}"),
                _ColouredColumnText(
                  title: Translations.of(context).totalReturns,
                  amount: widget.totalReturnAmount,
                  percent: widget.totalReturnPercent,
                ),
              ],
            ),
            const SizedBox(
              height: 15.0,
            ),
            ExpandedSection(
              expand: isExpanded,
              child: Column(
                children: [
                  Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      _ColumnText(
                          heading: Translations.of(context).currentInv,
                          //"Current Investment",
                          subtitle:
                              "₹ ${widget.currentInvestment != null ? indiaFormat.format(widget.currentInvestment).replaceAll("INR", "") : "--"}"),
                      _ColouredColumnText(
                        title:  Translations.of(context).currentReturns,

                        //"Current Returns",
                        amount: widget.currentReturnAmount,
                        percent: widget.currentReturnPercent,
                      ),
                    ],
                  ),
                  const SizedBox(
                    height: 8.0,
                  ),
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }
}

class _ColouredColumnText extends StatelessWidget {
  final String title;
  final num? percent;
  final num? amount;

  const _ColouredColumnText(
      {Key? key,
      required this.title,
      required this.percent,
      required this.amount})
      : super(key: key);
  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.end,
      children: [
        Row(
          children: [
            Text(
              title,
              style: const TextStyle(
                fontWeight: FontWeight.w200,
                fontSize: 11.0,
                color: Colors.grey,
              ),
            ),
            const SizedBox(
              width: 4.0,
            ),
            Text(
              "${percent ?? "--"} %",
              overflow: TextOverflow.ellipsis,
              style: TextStyle(
                  fontWeight: FontWeight.w400,
                  fontSize: 14.0,
                  color: percent != null && percent!.isNegative
                      ? Colors.red
                      : Colors.green),
            ),
          ],
        ),
        const SizedBox(
          height: 8.0,
        ),
        Text(
          "₹ ${amount != null ? indiaFormat.format(amount).replaceAll("INR", "") : "--"}",
          style: TextStyle(
              color: amount != null && amount!.isNegative
                  ? Colors.red
                  : Colors.green,
              fontWeight: FontWeight.w600,
              fontSize: 16.0),
        ),
      ],
    );
  }
}

class _ColumnText extends StatelessWidget {
  final String heading;
  final String subtitle;
  final CrossAxisAlignment crossAxisAlignment;
  const _ColumnText(
      {Key? key,
      required this.heading,
      required this.subtitle,
      this.crossAxisAlignment = CrossAxisAlignment.start})
      : super(key: key);
  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: crossAxisAlignment,
      children: [
        Text(
          heading,
          overflow: TextOverflow.ellipsis,
          style: const TextStyle(
            fontWeight: FontWeight.w200,
            fontSize: 11.0,
            color: Colors.grey,
          ),
        ),
        const SizedBox(
          height: 8.0,
        ),
        Text(
          subtitle,
          overflow: TextOverflow.ellipsis,
          style: const TextStyle(
            fontWeight: FontWeight.w500,
            fontSize: 16.0,
          ),
        ),
      ],
    );
  }
}
